#include "MESH_ConnectedComComputerMeshCoreVisitor.h"
#include "MESH_Vertex.h"
#include "MESH_SceneGraphTypes.h"
#include "MemReleaser.h"

#include <map>
#include <set>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

ConnectedComComputerMeshCoreVisitor::ConnectedComComputerMeshCoreVisitor()
{
  define(Recorder<Mesh>::getEnterFunction(&ConnectedComComputerMeshCoreVisitor::enterMesh));
}

void ConnectedComComputerMeshCoreVisitor::enterMesh(Mesh *M)
{
  ConnectedComComputerStateVariables *stateVariables = Singleton<ConnectedComComputerStateVariables>::getInstance();
  
  if (!stateVariables->getNode(M))
  {
    // computing the list of Mesh brothers of M.
    MFNode parentList = M->getVertex()->getParentList();
    
    // defining structures.
    std::vector<SFFace *> stack;
    const Mesh::MFFace &faces = M->getFaces();
    stack.reserve(faces.size());

    Mesh::MFFace::const_iterator itFace = M->getFaces().begin();
    
    while (itFace != faces.end())
    {
      std::set<SFFace *> faceSet;
      stack.push_back(*itFace);
      std::vector<SFFace *>::const_iterator f = stack.end();
      --f;
      faceSet.insert(*itFace);  
      
      while (f != stack.end())
      {
        const SFFace::MFEdge &me = (*f)->getEdges();
        for (SFFace::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        { 
          const SFEdge::MFFace &mlf = (*e)->getLeftFaces();
          const SFEdge::MFFace &mrf = (*e)->getRightFaces();
        
          for (SFEdge::MFFace::const_iterator lf = mlf.begin(); lf != mlf.end(); ++lf)
          {
            if (faceSet.find(*lf) == faceSet.end() && (faces.find(*lf) != faces.end()))
            {
              faceSet.insert(*lf); 
              stack.push_back(*lf);
            }
          }
        
          for (SFEdge::MFFace::const_iterator rf = mrf.begin(); rf != mrf.end(); ++rf)
          {
            if (faceSet.find(*rf) == faceSet.end() && (faces.find(*rf) != faces.end()))
            {
              faceSet.insert(*rf); 
              stack.push_back(*rf);
            }
          }
        }
        ++f;
      }

      // creating new Meshs and Vertex.
      Mesh *newMesh = new Mesh();
      newMesh->setChild(M->getVertex());

      // filling Mesh datas.
      newMesh->data().setNormal(M->data().hasNormal());
      newMesh->data().setNormalPerVertex(M->data().getNormalPerVertex());
      newMesh->data().setCreaseAngle(M->data().getCreaseAngle());

      newMesh->data().setColor(M->data().hasColor());
      newMesh->data().setColorPerVertex(M->data().getColorPerVertex());
      newMesh->data().setRGBA(M->data().getRGBA());

      newMesh->data().setTexCoord(M->data().hasTexCoord());

      newMesh->data().setSolid(M->data().isSolid());

      // iterating the faces of the connected component.
      for (std::set<SFFace *>::const_iterator it = faceSet.begin(); it != faceSet.end(); ++it)
      {
        MFInt32 faceCoordIndex;
        SFFace *face = *it;
        const SFFace::MFEdge &me = face->getEdges();
        SFFace::MFEdge::const_iterator e = me.begin();
        while (e != me.end())
        {
          faceCoordIndex.push_back((*e)->getFromVertex()->getIndex());
          ++e;
        }  

        SFFace *newFace = newMesh->createFace(faceCoordIndex);
        newFace->data().setNormal(face->data().getNormal());
        newFace->data().setColor(face->data().getColor());
        
        // filling vertex data.
        const SFFace::MFEdge &nme = newFace->getEdges();
        e = me.begin();
        SFFace::MFEdge::const_iterator ne = nme.begin();
        while (e != me.end())
        {
          SFVertex *v = (*e)->getFromVertex();
          SFVertex *newVertex = (*ne)->getFromVertex();
          
          newVertex->data().setNormalOfFace(v->data().getNormalOfFace(face), newFace);
          newVertex->data().setColorOfFace(v->data().getColorOfFace(face), newFace); 
          newVertex->data().setTexCoordOfFace(v->data().getTexCoordOfFace(face), newFace);  
          ++e;
          ++ne;  
        }
      }

      // adding new Meshes to the scene graph.
      MFNode parentList = M->getParentList();
      for (MFNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
      {
        SFNode cparent = (*itParent)->clone();
        cparent->setChild(newMesh);
          
        MFNode pparentList = (*itParent)->getParentList();
        for (MFNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
          (*itPParent)->setChild(cparent);
      }

      // finding the next face.
      while ((itFace != faces.end()) && (std::find(stack.begin(), stack.end(), *itFace) != stack.end()))
        ++itFace;
    }

    // remove old mesh.
    parentList = M->getParentList();
    for (MFNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
    {
      MFNode pparentList = (*itParent)->getParentList();
      for (MFNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
      {
        (*itPParent)->removeChild(*itParent);
        stateVariables->addToDeleteNode(*itParent);
      }
    }
    
    stateVariables->addNode(M);
  }
}

}
}  
