#include "X3D_BBoxUpdaterRenderingVisitor.h"
#include "X3D_Coordinate.h"
#include "X3D_X3DGeometryNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {


BBoxUpdaterRenderingVisitor::BBoxUpdaterRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  define(Recorder<Coordinate>::getEnterFunction(&BBoxUpdaterRenderingVisitor::enterCoordinate));

  // WalkOn functions.
  define(Recorder<X3DGeometryNode>::getWalkOnFunction(&BBoxUpdaterRenderingVisitor::walkOnX3DGeometryNode));
}

void BBoxUpdaterRenderingVisitor::enterCoordinate(Coordinate *C) 
{
  BBoxUpdaterStateVariables *stateVariables = Singleton<BBoxUpdaterStateVariables>::getInstance();

  BBox *BB = stateVariables->getBBox(C);
  if (BB == 0)
  {
    const MFVec3f &vertexArray = C->getPoint();
  
    if (vertexArray.empty())
    {
      BB = new BBox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(-1.0f, -1.0f, -1.0f));
      return;
    }
  
    SFVec3f min = vertexArray[0];
    SFVec3f max = vertexArray[0];
  
    for (MFVec3f::const_iterator itVertex = vertexArray.begin(); itVertex != vertexArray.end(); ++itVertex)
    {
      if ((*itVertex).x < min.x)
        min.x = (*itVertex).x;
      if ((*itVertex).y < min.y)
        min.y = (*itVertex).y;
      if ((*itVertex).z < min.z)
        min.z = (*itVertex).z;
   
      if ((*itVertex).x > max.x)
        max.x = (*itVertex).x;
      if ((*itVertex).y > max.y)
        max.y = (*itVertex).y;
      if ((*itVertex).z > max.z)
        max.z = (*itVertex).z;
    }
  
    //computing the new center
    SFVec3f center = 0.5f*(min + max);
        
    BB = new BBox(center, 2.0f*(max - center));  
    stateVariables->addBBox(C, BB);
    stateVariables->setShapeBBox(*BB);    
  }
}

bool BBoxUpdaterRenderingVisitor::walkOnX3DGeometryNode(X3DGeometryNode *, SFNode) 
{
  return true;
}

}
}
